﻿using System;
using System.Collections.Generic;
using Ozeki.Media;
using Ozeki.Network;
using Ozeki.VoIP;

namespace _04_Multi_Level_IVR
{
    class Program
    {
        static List<CallHandler> _callHandlers;

        static string _blindTransfer;

        static void Main(string[] args)
        {
            _callHandlers = new List<CallHandler>();
            var softphone = new Softphone();

            ShowHelp();

            Console.WriteLine("Please set the number for blind transferring! If you don't want to use this function of the IVR, press 0!");
            _blindTransfer = Read("blindTransfer", true);

            SipAccountInitialization(softphone);

            softphone.IncomigCall += softphone_IncomigCall;
            Console.ReadLine();
        }

        private static void ShowHelp()
        {
            Console.ForegroundColor = ConsoleColor.Red;

            Console.WriteLine("\nThis is an example code for Interactive Voice Response (IVR) written in C#. This is a technology that allows a computer to interact with humans through the use of voice and DTMF tones input via keypad.");
            Console.WriteLine("You will need to  setup your SIP account (your authentication ID, username, display name etc.). If the registration succeeded and you call the number of the IVR you will hear a greeting message.");
            Console.WriteLine("After that you can press the button one to hear more information about Ozeki Ltd., and if you press button two you will hear a sample mp3 file. By pressing button three, you can enter a lower menu level. In this level you can transfer the call to the added number if you press button one. By pressing button two, you can return to the main menu.\n");

            Console.ForegroundColor = ConsoleColor.White;

        }

        static void softphone_IncomigCall(object sender, VoIPEventArgs<Ozeki.VoIP.IPhoneCall> e)
        {
            Console.WriteLine("Incoming call!");
            var callHandler = new CallHandler(e.Item);
            callHandler.BlindTransferNumber(_blindTransfer);

            callHandler.Completed += callHandler_Completed;

            lock (_callHandlers)
                _callHandlers.Add(callHandler);

            callHandler.Start();
        }

        static void callHandler_Completed(object sender, EventArgs e)
        {
            lock (_callHandlers)
                _callHandlers.Remove((CallHandler)sender);
        }

        private static void SipAccountInitialization(Softphone softphone)
        {
            Console.WriteLine("Please setup your SIP account!\n");
            Console.WriteLine("Please set your authentication ID: ");
            var authenticationId = Read("authenticationId", true);

            Console.WriteLine("Please set your user name (default:" + authenticationId + "): ");
            var userName = Read("userName", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            Console.WriteLine("Please set your name to be displayed (default: " + authenticationId + "): ");
            var displayName = Read("displayName", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            Console.WriteLine("Please set your registration password: ");
            var registrationPassword = Read("registrationPassword", true);

            Console.WriteLine("Please set the domain name (default: your local host): ");
            var domainHost = Read("domainHost", false);
            if (string.IsNullOrEmpty(domainHost))
                domainHost = NetworkAddressHelper.GetLocalIP().ToString();

            Console.WriteLine("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("domainPort", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = int.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register...\n");
            softphone.Register(true, displayName, userName, authenticationId, registrationPassword, domainHost, domainPort);
        }

        private static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                {
                    return input;
                }

                if (!string.IsNullOrEmpty(input))
                {
                    return input;
                }

                Console.WriteLine(inputName + " cannot be empty!");
                Console.WriteLine(inputName + ": ");
            }
        }
    }
}
